#!/usr/bin/perl

## ---------------------------------------------------------------------
#
# Author: Xavier Noria <`echo -n ska@unfuers.pbz | tr a-z n-za-m`>
# Time-stamp: <2003-09-26 23:34:59 fxn>
#
## ---------------------------------------------------------------------

###
### Modules ------------------------------------------------------------
###

use strict;
use warnings;

use vars qw($VERSION $from_regexp $to_regexp $subject_regexp);
$VERSION = '1.3';

use Getopt::Std;
use Net::POP3;
use Term::ANSIColor qw(:constants);


###
### Utility subroutines ------------------------------------------------
###

sub say { print @_, "\n" }


###
### Parameters processing ----------------------------------------------
###

# Process parameters.
my %params = ();
getopts('vns:l:p:', \%params);

exists $params{v} and print <<VERSION and exit;
SwenDeleter $VERSION, by fxn.
VERSION

my ($server, $login, $password) = @params{qw(s l p)};
defined $server and defined $login and defined $password or print <<USAGE and exit;
Usage: $0 -s server -l login -p password [-n]
USAGE

my $nonstop = exists $params{n};


###
### Filtering ----------------------------------------------------------
###

# We use the worm's size as a threshold.
my $swen_size = 106496; # bytes

# Connect to the server.
my $pop;
eval { $pop = Net::POP3->new($server) };
defined $pop or print <<MSG and exit;
Couldn't connect to the POP3 server $server.
MSG

# Log in. Do not print login data to the screen if not possible.
my $nmails;
eval { $nmails = $pop->login($login, $password) };
defined $nmails or print <<MSG and exit;
I could connect to the server but couldn't log in.
MSG

# In the trials I've done $nmails is "0E0" (which is true in Perl) if
# there are no new mails, or a number otherwise. That's why an explicit
# numerical test is needed.
unless ($nmails > 0) {
    say "No mail.";
    exit;
}

# Construct $(from|to|subject)_regexp.
init_regexps();

my @suspicious  = ();
my %messages = %{$pop->list};
while (my ($msgnum, $size) = each %messages) {
    next if $size < $swen_size; # skip the message unless it's big enough

    # Get the headers.
    my @lines = @{$pop->top($msgnum)};

    # Extract relevant headers.
    my ($from)    = grep /^From:/i, @lines;
    my ($to)      = grep /^To:/i, @lines;
    my ($subject) = grep /^Subject:/i, @lines;

    # Some of those headers do not exist sometimes, we interpret that as
    # an indication of infection.
    my $matches = 0;
    ++$matches if not defined $from    or $from    =~ $from_regexp;
    ++$matches if not defined $to      or $to      =~ $to_regexp;
    ++$matches if not defined $subject or $subject =~ $subject_regexp;
    if ($matches >= 2) {

        # Highlight match if any, unless this is Windows.
        unless ($^O eq 'MSWin32') { # according to perlvar this eq-test is right, /win/i matches "darwin"
            $from    =~ s/$from_regexp/BOLD . RED . $& . RESET/geo    if defined $from;
            $to      =~ s/$to_regexp/BOLD . RED . $& . RESET/geo      if defined $to;
            $subject =~ s/$subject_regexp/BOLD . RED . $& . RESET/geo if defined $subject;
        }

        # We print this even in nonstop mode for logging purposes.
        print "\nSize: @{[int($size/1024)]}K\n";
        print $from, $to, $subject;

        push(@suspicious, $msgnum), next if $nonstop;

        my $a;
        do {
            print 'Mark message for deletion? ([Y]es, [n]o, [a]bort): ';
            $a = <STDIN>
        } until $a =~ /^[yna]?$/i;
        if ($a =~ /^y?$/i) {
            push @suspicious, $msgnum;
        } elsif ($a =~ /^a$/i) {
            say "Aborted. No messages were deleted from the server.";
            $pop->quit;
            exit;
        }
    }
}

if (@suspicious) {
    unless ($nonstop) {
        my $a;
        do {
            print "\nAll done. Delete marked messages from the server? ([Y]es, [n]o): ";
            $a = <STDIN>;
        } until $a =~ /^[yn]?$/i;
        if ($a =~ /^n$/i) {
            say "Aborted. No messages were deleted from the server.";
            $pop->quit;
            exit;
        }
    }

    my $total_size = 0;
    foreach my $msgnum (@suspicious) {
        $total_size += $messages{$msgnum};
        $pop->delete($msgnum);
        --$nmails;
    }
    say "@{[scalar @suspicious]} deleted, total size was: @{[int($total_size/1024)]}K";
}

$pop->quit;
say "$nmails left in the mailbox";



# The regexps are constructed taking into account these studies:
#
#      http://www.f-secure.com/v-descs/swen.shtml
#      http://www.pandasoftware.es/virus_info/enciclopedia/verficha.aspx?lst=det&idvirus=40743
#
# This subroutine sets the globals $(from|to|subject)_regexp. We have
# factored it out to clean the main code.
sub init_regexps {

    # Froms can follow two patters:
    my $from1_name = join "|", qw(
MS
Microsoft
Corporation
Program
Internet
Network
Security
Division
Section
Department
Center
Technical
Public
Customer
Bulletin
Services
Assistance
Support
);
    my $from1_name_regexp = qr/(?:$from1_name)(?:\s+(?:$from1_name))*/i;

    my $from1_address_subdomain = join "|", qw(
support
technet
updates
advisor
confidence
bulletin
news?
newsletters?
);

    my $from1_address_domain = join "|", qw(
ms
msdn
msn
microsoft
);

    my $from1_address_tld = join "|", qw(
net
com
);

    my $from1_address_regexp = qr/\@(?:
                                      (?:$from1_address_subdomain)
                                      [._]
                                      (?:$from1_address_domain)
                                      |
                                      (?:$from1_address_subdomain)
                                      |
                                      (?:$from1_address_domain))
                                  \.(?:$from1_address_tld)/xi;

    my $from1_regexp = qr/$from1_name_regexp.*$from1_address_regexp
                          |
                          $from1_name_regexp
                          |
                          $from1_address_regexp/xi;

    my $from2_name = join "|", qw(
Admint?
Administrator
Postmaster
Delivery
Email
Inet
Internet
Mail
Message
Net
Network
Service
Storage
System
);

    my $from2_name_regexp = qr/(?:$from2_name)(?:\s+(?:$from2_name))*/i;

    my $from2_address_user_first = join "|", qw(
email
mailer
master
post
smtp
web
);

    my $from2_address_user_last = join "|", qw(
automat
bot
daemon
engine
form
program
robot
routine
service
);

    my $from2_address_domain = join "|", qw(
America
AOL
bigfoot
freemail
microsoft
netmail
puremail
rocketmail
visitmail
yahoo
);

    my $from2_address_tld = join "|", qw(
net
com
);

    my $from2_address_regexp = qr/\w?
                                  (?:$from2_address_user_first)
                                  (?:$from2_address_user_last)
                                  \@
                                  .*
                                  (?:$from2_address_domain)
                                  \.
                                  (?:$from2_address_tld)/xi;

    my $from2_regexp = qr/$from2_name_regexp.*$from2_address_regexp
                          |
                          $from2_name_regexp
                          |
                          $from2_address_regexp/xi;

    $from_regexp = qr/$from1_regexp|$from2_regexp|"\s*"|<\s*>/i;

    # There can be two kind of subjects as well.
    my $subject1_words = join "|", qw(
Current
Newest
Last
New
Latest
Net
Network
Microsoft
Internet
Critical
Security
Patch
Update
Pack
Upgrade
E?mail
);
    my $subject1_regexp = qr/(?:$subject1_words)(?:\s+(?:$subject1_words))*/i;

    my $subject2_regexp =
      qr/(?:(?:abort|bug|error|failure)\s+)?(?:advice|announcement|letter|message|notice|report)
         |
         (?:(?:(?:returned|undelivered|undeliverable)\s+(?:mail|message|)\s*:\s*)?
         (?:returned\s+to\s+sender|returned\s+to\s+mailer|user\s+unknown))/xi;

    $subject_regexp = qr/$subject1_regexp|$subject2_regexp|^Subject:\s*$/i;

    my $to_words = join "|", qw(
Commercial
MS
Microsoft
Corporation
Customer
User
Partner
Consumer
Client
);
    $to_regexp = qr/(?:$to_words)(?:\s*(?:$to_words))*|"\s*"|<\s*>/i;

}

__END__
